/*	ESP32 Connections
	Screen:						MP3 Decoder:			Buttons:
	DATA			PIN		|	DATA			PIN		|	DATA			PIN
	LED				17		|	5V				5V		|	BW (Backward)	22
	SCK				14		|	DGND			GND		|	PRESET1			21
	SDA				13		|	MOSI			23		|	PRESET2			32
	A0				2		|	DREQ			4		|	PRESET3			33
	RESET			27		|	XCS				5		|	PRESET4			25
	CS				15		|	5V				5V		|	FW (Forward)	26
	GND				GND		|	MISO			19		|	IR:
	VCC				5V		|	SCK				18		|	VCC				5V
							|	XRST			EN		|	GND				GND
							|	XDCS			16		|	Data			34
*/

#include <esp_wifi.h>
#include <IRremote.h>
#include <Preferences.h>
#include <vs1053_ext.h>
#include <TFT_eSPI.h>
#include <Fonts\GFXFF\C__Windows_Fonts_calibrib6pt7b.h>
#include <Free_Fonts.h>

// MP3 Decoder Pins
#define SPI_MOSI		23
#define SPI_MISO		19
#define SPI_SCK			18
#define VS1053_CS		5
#define VS1053_DCS		16
#define VS1053_DREQ		4

int IR_RECEIVE_PIN = 34;

VS1053 player(VS1053_CS, VS1053_DCS, VS1053_DREQ);
TFT_eSPI tft = TFT_eSPI();
TFT_eSprite scrolltxt_station	= TFT_eSprite(&tft);
TFT_eSprite scrolltxt_stream	= TFT_eSprite(&tft);
Preferences preferences;

// Put here your set of SSID and Passwords. Leave the first one empty in both cases
const char *ssid[] = {" ","WiFi1","WiFi2","WiFi3","WiFi4"};
const char *password[] = {" ","Pass1","Pass2","Pass3","Pass4"};

// Icons
#define wifiicon_width 12
#define wifiicon_height 12
PROGMEM const unsigned char wifiicon[] = {
	0x00, 0x00, 0x00, 0x00, 0xF8, 0x01, 0x0E, 0x07, 0x03, 0x0C, 0xF8, 0x01, 
	0x0C, 0x03, 0x00, 0x00, 0x60, 0x00, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 
	};

#define nowifiicon_width 12
#define nowifiicon_height 12
PROGMEM const unsigned char nowifiicon[] = {
  0xF0, 0x00, 0x08, 0x01, 0x04, 0x02, 0x02, 0x05, 0x81, 0x08, 0x41, 0x08, 
  0x21, 0x08, 0x11, 0x08, 0x0A, 0x04, 0x04, 0x02, 0x08, 0x01, 0xF0, 0x00, 
  };

#define saveicon_width 12
#define saveicon_height 12
PROGMEM const unsigned char saveicon[] = {
	0x00, 0x00, 0xFE, 0x01, 0x4A, 0x03, 0x7A, 0x06, 0x02, 0x04, 0x02, 0x04, 
	0x02, 0x04, 0xFA, 0x05, 0x0A, 0x05, 0x0A, 0x05, 0xFE, 0x07, 0x00, 0x00, 
	};

char *new_radio;
char *radiopresets[] =
{
	// Jazz
	"http://185.33.21.112:80/ajazz_64",						// #01 1.FM - Adore Jazz Radio
	"http://listen.radionomy.com/ABC-Jazz",					// #02 ABC Jazz
	"http://tgas.dyndns.org:8001/1",						// #03 The Great American Songbook
	"https://listen.57fm.com:80/toj",						// #04 A Taste of Jazz
	"http://audio-edge-es6pf.mia.g.radiomast.io/f0ac4bf3-bbe5-4edb-b828-193e0fdc4f2f",	// #05 Laid Back Jazz
	"http://195.154.79.204:8638/stream",					// #06 ALR Jazz Radio
	"http://195.201.166.244:8270/stream",					// #07 Fascinating Jazz Radio
	"https://kathy.torontocast.com:3330/",					// #08 Jazz Sakura
	"http://lyd.nrk.no/nrk_radio_jazz_mp3_h",				// #09 NRK Jazz
	"http://94.23.159.187:9998/stream",						// #10 Frank Sinatra Radio
	"http://213.32.121.35:8060/radio.mp3",					// #11 Best Jazz
	"http://66.147.225.141:8021/stream",					// #12 Crooner Radio Online
	"http://75.102.53.58/1067",								// #13 113.fm Jazzy!
	"http://62.210.204.2:8000/stream/5/",					// #14 A A Jazz
	"http://listen.radionomy.com/ABC-Love",					// #15 ABC Love
	"http://manager7.streamradio.fr:1050/",					// #16 ABC Relax New
	"https://mpc1.mediacp.eu:8404/stream",					// #17 Radio Dimensione Jazz
	
	// Smooth Jazz
	"http://75.102.53.58/1066",								// #18 113.fm The Wave
	"http://77.68.84.201/stream/",							// #19 Relaxing Jazz
	"http://185.33.21.112/sax4ever_128",					// #20 1.FM - Sax4Ever
	"http://51.255.235.165:5092/stream",					// #21 Qfm Cool Jazz HD
	"http://64.95.243.43:8002/stream",						// #22 Best Smooth Jazz UK
	"http://streams.crooze.fm:8002/stream128",				// #23 Jazz Crooze
	
	// Lounge
	"http://65.108.124.70:9328/stream",						// #24 rtpONE Chillout Lounge
	"http://185.33.21.111/chilloutlounge_128",				// #25 1.FM - Chillout Lounge Radio
	"http://maggie.torontocast.com:8028/stream",			// #26 Sensual Lounge
	"http://50.31.185.139/1064",							// #27 113.fm The Lounge
	"http://64.95.243.43:8020/stream",						// #28 Smooth Grooving UK
	"http://107.150.53.50:6226/stream",						// #29 Roma
	"http://216.235.80.70:80/2586_128.mp3",					// #30 Smooth Lounge
	
	// Funk & Soul
	"http://176.31.111.65:4744",							// #31 Funky Corner Radio
	"http://91.121.104.123:8000/stream/2/",					// #32 Generation Soul Disco Funk Radio
	"http://216.235.89.134/bumpin",							// #33 Power Hitz
	"http://88.202.190.66:8330/stream",						// #34 Solar Radio
	"http://185.33.21.112/funkyexpress_128",				// #35 1.FM Funky Express
	
	// Bossa Nova
	"http://65.108.124.70:9242/stream",						// #36 rtpONE LATIN BOSSA NOVA
	"http://185.33.21.112/bossanova_128",					// #37 1.FM - Bossa Nova Hits
	"http://usa10.fastcast4u.com:3720/",					// #38 Bra - Blue Bossa Nova Radio
	
	// Pop 80's
	"http://airspectrum.cdnstream1.com:8000/1261_192",		// #39 Magic Oldies Florida
	"http://s1-webradio.antenne.de/oldies-but-goldies",		// #40 Oldie Antenne
	"http://46.4.40.246:9199/stream",						// #41 Oldies 100
	
	// Otros
	"http://stream.live.vc.bbcmedia.co.uk/bbc_radio_two",	// #42 BBC Radio 2
	
	// Música Clásica
	"http://5.135.173.165/stream1",							// #43 Venice Classic Radio Italia
	"http://51.68.215.9:8000/radio.mp3"						// #44 Classical for Studying
};

String station_text;
String stream_text;

int radio_no;
int volume		= 21;
int tcount1		= 0;
int tcount2		= 0;
int radio_qty	= sizeof(radiopresets) / sizeof(radiopresets[0]);
int length_station;
int length_stream;
int wifi_index = 0;
int key_qty	= sizeof(ssid) / sizeof(ssid[0]);
int no_conn = 0;

unsigned long currentMillis1 = 0;
unsigned long previousMillis1 = 0;
unsigned long currentMillis2 = 0;
unsigned long previousMillis2 = 0;
unsigned long currentMillisWiFi = 0;
unsigned long previousMillisWiFi = 0;
unsigned long currentMillisWiFi2 = 0;
unsigned long previousMillisWiFi2 = 0;

const int BW_PIN = 22;	// Pin for 'Back' Button
const int P1_PIN = 21;	// Pin for 'Preset 1' Button
const int P2_PIN = 32;	// Pin for 'Preset 2' Button
const int P3_PIN = 33;	// Pin for 'Preset 3' Button
const int P4_PIN = 25;	// Pin for 'Preset 4' Button
const int FW_PIN = 26;	// Pin for 'Forward' Button
const int SHORT_PRESS_TIME = 500;  // Limit time frame for short/long button press

int lastState_BW = HIGH;  // Previous state for 'Back' Button
int currentState_BW;  // Current state for 'Back' Button
unsigned long pressedTime_BW  = 0;
unsigned long releasedTime_BW = 0;

int lastState_FW = HIGH;  // Previous state for 'Forward' Button
int currentState_FW;  // Current state for 'Forward' Button
unsigned long pressedTime_FW  = 0;
unsigned long releasedTime_FW = 0;

int lastStatePreset[4] = {HIGH,HIGH,HIGH,HIGH};  // Previous state for 'Preset X' Button
int currentStatePreset[4];  // Current state for 'Preset X' Button
unsigned long pressedTimePreset[4]  = {0,0,0,0};
unsigned long releasedTimePreset[4] = {0,0,0,0};

void setup()
{
	esp_sleep_enable_ext0_wakeup(GPIO_NUM_26,0);  // Turn On Streamer with Forward Button
	gpio_hold_dis(GPIO_NUM_17);  // This prevents black screen after wake up
	
	pinMode(BW_PIN, INPUT_PULLUP);
	pinMode(P1_PIN, INPUT_PULLUP);
	pinMode(P2_PIN, INPUT_PULLUP);
	pinMode(P3_PIN, INPUT_PULLUP);
	pinMode(P4_PIN, INPUT_PULLUP);
	pinMode(FW_PIN, INPUT_PULLUP);
	
	pinMode(IR_RECEIVE_PIN, INPUT);
	
	pinMode(17, OUTPUT);
	digitalWrite(17, HIGH);
	digitalWrite(0, HIGH);
	
	preferences.begin("Save/Load", false);
	IrReceiver.begin(IR_RECEIVE_PIN, DISABLE_LED_FEEDBACK);
	SPI.begin(SPI_SCK, SPI_MISO, SPI_MOSI);
	tft.init();
	setScreen();
	player.begin();
	delay(1000);
	player.setVolume(volume);
	
	Serial.begin(115200);
	delay(100);
	Serial.println("Shadowilla Design");
	Serial.println("Internet Radio Streaming");
	Serial.println();
	
	connectToWiFinStreamer();
}

void loop()
{
	player.loop();
	
	// Scroll Station
	if(length_station > 23)
	{
		currentMillis1 = millis();
		if (currentMillis1 - previousMillis1 >= 30)
		{
			previousMillis1 = currentMillis1;
			scrolltxt_station.pushSprite(10,70);  // Sets Sprite on screen (x,y)
			scrolltxt_station.scroll(-1);  // Scroll direction
			tcount1--;
			if (tcount1 <= 0)
			{
				tcount1 = length_station*7.5;  // 7.5 factor was the best for this size of screen in terms of spacing between sprites
				scrolltxt_station.drawString(station_text, 160, 0); // Draws word in position (x,y)
			}
		}
	}
	
	// Scroll Stream
	if(length_stream > 23)
	{
		currentMillis2 = millis();
		if (currentMillis2 - previousMillis2 >= 30)
		{
			previousMillis2 = currentMillis2;
			scrolltxt_stream.pushSprite(10,100);  // Sets Sprite on screen (x,y)
			scrolltxt_stream.scroll(-1);  // Scroll direction
			tcount2--;
			if (tcount2 <= 0)
			{
				tcount2 = length_stream*7.5;  // 7.5 factor was the best for this size of screen in terms of spacing between sprites
				scrolltxt_stream.drawString(stream_text, 160, 0); // Draws word in position (x,y)
			}
		}
	}
	
	// Check 'Buttons' status
	currentState_BW = digitalRead(BW_PIN);
	currentStatePreset[0] = digitalRead(P1_PIN);
	currentStatePreset[1] = digitalRead(P2_PIN);
	currentStatePreset[2] = digitalRead(P3_PIN);
	currentStatePreset[3] = digitalRead(P4_PIN);
	currentState_FW = digitalRead(FW_PIN);
	
	checkBWButton(currentState_BW,lastState_BW);
	checkPresetButton(currentStatePreset[0],lastStatePreset[0],0);
	checkPresetButton(currentStatePreset[1],lastStatePreset[1],1);
	checkPresetButton(currentStatePreset[2],lastStatePreset[2],2);
	checkPresetButton(currentStatePreset[3],lastStatePreset[3],3);
	checkFWButton(currentState_FW,lastState_FW);
	
	lastState_BW = currentState_BW;
	lastStatePreset[0] = currentStatePreset[0];
	lastStatePreset[1] = currentStatePreset[1];
	lastStatePreset[2] = currentStatePreset[2];
	lastStatePreset[3] = currentStatePreset[3];
	lastState_FW = currentState_FW;
	
	// Check for IR commands
	checkIR();
	
	// Check for Serial input of radios (link or radio number)
	checkSerialMonitor();
	
	// Check WiFi status every 1 minute
	currentMillisWiFi = millis();
	if (currentMillisWiFi - previousMillisWiFi >= 60000)
	{
		previousMillisWiFi = currentMillisWiFi;
		checkWiFi();
	}
}

void setScreen(void)
{
	tft.setRotation(1);
	tft.fillScreen(TFT_BLACK);
	tft.setFreeFont(OWNFONT51);
	tft.setTextColor(tft.color565(0,255,255),TFT_BLACK);
	tft.setCursor(50,60);
	tft.print("Welcome!");
	delay(3000);
	
	tft.fillScreen(TFT_BLACK);
	tft.setTextColor(TFT_YELLOW,TFT_BLACK);
	tft.setCursor(5,20);
	tft.print("Internet Streaming");
	
	tft.setTextColor(tft.color565(0,255,255),TFT_BLACK);
	tft.setCursor(5,32);
	tft.print("Shadowilla Design");
	
	tft.drawLine(0,39,160,39,TFT_WHITE);
	
	tft.setTextColor(TFT_WHITE,TFT_BLACK);
	tft.setCursor(135,51);
	tft.print("kbps");
	tft.setCursor(10,65);
	tft.print("Radio:");
	tft.setCursor(10,95);
	tft.print("Stream:");
	
	tft.setTextColor(tft.color565(0,255,0),TFT_BLACK);
	
	scrolltxt_station.setFreeFont(OWNFONT51);
	scrolltxt_station.setColorDepth(8);
	scrolltxt_station.createSprite(750, 11);
	scrolltxt_station.fillSprite(TFT_BLACK);
	scrolltxt_station.setTextColor(tft.color565(0,255,0),TFT_BLACK);
	
	scrolltxt_stream.setFreeFont(OWNFONT51);
	scrolltxt_stream.setColorDepth(8);
	scrolltxt_stream.createSprite(750, 11);
	scrolltxt_stream.fillSprite(TFT_BLACK);
	scrolltxt_stream.setTextColor(tft.color565(0,255,0),TFT_BLACK);
}

void textsReset(void)
{
	scrolltxt_station.fillSprite(TFT_BLACK);
	scrolltxt_stream.fillSprite(TFT_BLACK);
	tft.fillRect(0,70,160,11, TFT_BLACK);
	tft.fillRect(0,100,160,11, TFT_BLACK);
	tft.fillRect(115,44,20,11, TFT_BLACK);
	tft.fillRect(55,58,105,11, TFT_BLACK);
	tcount1=0;
	tcount2=0;
	station_text = "";
	stream_text = "";
	delay(250);
}

void connectToWiFinStreamer(void)
{
	WiFi.setSleep(false);
	WiFi.mode(WIFI_STA);
	esp_wifi_set_ps(WIFI_PS_NONE);
	
	WiFi.begin(ssid[wifi_index], password[wifi_index]);
	Serial.print("Connecting to: ");
	Serial.println(ssid[wifi_index]);
	while (WiFi.status() != WL_CONNECTED)
	{
		Serial.print(".");
		delay(500);
		currentMillisWiFi2 = millis();
		if (currentMillisWiFi2 - previousMillisWiFi2 >= 5000)  // If 5 secconds pass, connect to next network
		{
			previousMillisWiFi2 = currentMillisWiFi2;
			wifi_index += 1;
			if (wifi_index >= key_qty)
			{
				wifi_index = 0;
				break;
			}
			WiFi.begin(ssid[wifi_index], password[wifi_index]);
			Serial.println();
			Serial.print("Connecting to new network: ");
			Serial.println(ssid[wifi_index]);
		}
	}
	
	if(WiFi.status() == WL_CONNECTED)
	{
		tft.drawXBitmap(145, 12, wifiicon, wifiicon_width, wifiicon_height, TFT_BLACK, TFT_BLACK);
		tft.drawXBitmap(145, 12, wifiicon, wifiicon_width, wifiicon_height, TFT_WHITE, TFT_BLACK);
		Serial.println();
		Serial.print("Successfuly connected to: ");
		Serial.println(ssid[wifi_index]);
		wifi_index -= 1;
		currentMillisWiFi2 = 0;
		previousMillisWiFi2 = 0;
		no_conn = 0;
		
		radio_no = preferences.getInt("radio_no"+99, radio_no);
		if (radio_no < 0 || radio_no > radio_qty-1) {radio_no = 0;}
		connectiontoRadio();
	}
	else if(WiFi.status() != WL_CONNECTED)
	{
		tft.drawXBitmap(145, 12, wifiicon, wifiicon_width, wifiicon_height, TFT_BLACK, TFT_BLACK);
		tft.drawXBitmap(145, 12, nowifiicon, nowifiicon_width, nowifiicon_height, TFT_RED, TFT_BLACK);
		Serial.println();
		Serial.println("Unable to connect to WiFi");
		wifi_index = 0;
		currentMillisWiFi2 = 0;
		previousMillisWiFi2 = 0;
		if (no_conn == 3)	{offRoutine();}  // If can't connect after 3 attempts, turn Streamer off
		else				{no_conn += 1;}
	}
}

void connectiontoRadio(void)
{
	textsReset();
	new_radio = radiopresets[radio_no];
	player.connecttohost(new_radio);
	Serial.print("Radio n° ");
	Serial.println(radio_no+1);
}

void checkWiFi(void)
{
	if (WiFi.status() != WL_CONNECTED)
	{
		Serial.println("No WiFi. Reconnecting...");
		textsReset();
		tft.drawXBitmap(145, 12, wifiicon, wifiicon_width, wifiicon_height, TFT_BLACK, TFT_BLACK);
		tft.drawXBitmap(145, 12, nowifiicon, nowifiicon_width, nowifiicon_height, TFT_RED, TFT_BLACK);
		connectToWiFinStreamer();
	}
}

void vs1053_showstation(const char *info)
{
	Serial.print("STATION:      ");
	Serial.println(info);
	
	station_text = String(info);
	length_station = station_text.length();
	scrolltxt_station.fillSprite(TFT_BLACK);
	tft.fillRect(0,70,160,11, TFT_BLACK);
	tft.setCursor(10,78);
	tcount1=0;
	if		(length_station < 2)							{tft.print("No data");}	// If no text, show "No data"
	else if	(length_station >=2 && length_station <= 23)	{tft.print(station_text);}	// If there's text and is short, don't scroll
	
	tft.fillRect(55,58,105,11, TFT_BLACK);
	tft.setCursor(55,65);
	tft.print(radio_no+1);
}

void vs1053_showstreamtitle(const char *info)
{
	Serial.print("STREAMTITLE:  ");
	Serial.println(info);
	
	stream_text = String(info);
	length_stream = stream_text.length();
	scrolltxt_stream.fillSprite(TFT_BLACK);
	tft.fillRect(0,100,160,11, TFT_BLACK);
	tft.setCursor(10,108);
	tcount2=0;
	if		(length_stream < 2)							{tft.print("No data");}	// If no text, show "No data"
	else if	(length_stream >=2 && length_stream <= 23)	{tft.print(stream_text);}	// If there's text and is short, don't scroll
}

void vs1053_bitrate(const char *br)
{
	Serial.print("BITRATE:      ");
	Serial.println(String(br)+" kbps");
	
	tft.fillRect(115,44,20,11, TFT_BLACK);
	tft.setCursor(115,51);
	tft.print(br);
}

void checkBWButton(int curState, int lasState)
{
	// Events for 'Back' Button
	if (lasState == HIGH && curState == LOW)  // 'Back' Button is pressed
		pressedTime_BW = millis();
	
	else if (lasState == LOW && curState == HIGH)  // 'Back' Button is released
	{
		releasedTime_BW = millis();
		if (releasedTime_BW - pressedTime_BW < SHORT_PRESS_TIME)  // If it's a 'Short' press: Go back 1 station
		{
			Serial.println("Short press: 'Back'");
			radio_no--;
			if (radio_no < 0 || radio_no > radio_qty-1)	{radio_no = radio_qty-1;}  // If array index is out of bounds, return to the end of the array
			connectiontoRadio();
		}
		else if (releasedTime_BW - pressedTime_BW >= SHORT_PRESS_TIME && releasedTime_BW - pressedTime_BW <= 2000)  // If it's a 'Semilong' press: Go back 5 stations
		{
			Serial.println("Semilong press: 'Back'");
			radio_no-=5;
			if (radio_no < 0 || radio_no > radio_qty-1)	{radio_no = radio_qty-1;}  // If array index is out of bounds, return to the end of the array
			connectiontoRadio();
		}
		else if (releasedTime_BW - pressedTime_BW > 2000)  // If it's a 'Long' press: Shutdown
		{
			Serial.println("Long press: 'Back'");
			offRoutine();
		}
	}
}

void checkFWButton(int curState, int lasState)
{
	// Events for 'Forward' Button
	if (lasState == HIGH && curState == LOW)  // 'Forward' Button is pressed
		pressedTime_FW = millis();
	
	else if (lasState == LOW && curState == HIGH)  // 'Forward' Button is released
	{
		releasedTime_FW = millis();
		if (releasedTime_FW - pressedTime_FW < SHORT_PRESS_TIME)  // If it's a 'Short' press: Go forward 1 station
		{
			Serial.println("Short press: 'Forward'");
			radio_no++;
			if (radio_no < 0 || radio_no > radio_qty-1)	{radio_no = 0;}  // If array index is out of bounds, return to the beginning of the array
			connectiontoRadio();
		}
		else   // If it's a 'Long' press: Go forward 5 stations
		{
			Serial.println("Long press: 'Forward'");
			radio_no+=5;
			if (radio_no < 0 || radio_no > radio_qty-1)	{radio_no = 0;}  // If array index is out of bounds, return to the beginning of the array
			connectiontoRadio();
		}
	}
}

void checkPresetButton(int curState, int lasState, int index)
{
	// Events for 'Preset x' Button
	if (lasState == HIGH && curState == LOW)  // 'Preset x' Button is pressed
		pressedTimePreset[index] = millis();
	
	else if (lasState == LOW && curState == HIGH)  // 'Preset x' Button is released
	{
		releasedTimePreset[index] = millis();
		if (releasedTimePreset[index] - pressedTimePreset[index] < SHORT_PRESS_TIME)  // If it's a 'Short' press: Go to saved radio on that Preset
		{
			Serial.print("Short press Preset: ");
			Serial.println(index+1);
			radio_no = preferences.getInt("radio_no"+index, radio_no);
			if (radio_no < 0 || radio_no > radio_qty-1) {radio_no = 0;}  // If array index is out of bounds, go to first radio listed
			connectiontoRadio();
		}
		else  // If it's a 'Long' press: Save radio in selected Preset
		{
			Serial.print("Long press Preset: ");
			Serial.println(index+1);
			saveRadioMem(index);
		}
	}
}

void saveRadioMem(int index)
{
	if (radio_no < 0 || radio_no > radio_qty-1) {radio_no = 0;}
	preferences.putInt("radio_no"+index, radio_no);
	tft.drawXBitmap(130, 12, saveicon, saveicon_width, saveicon_height, TFT_WHITE, TFT_BLACK);
	delay(750);
	tft.drawXBitmap(130, 12, saveicon, saveicon_width, saveicon_height, TFT_BLACK, TFT_BLACK);
	Serial.print("Radio n° ");
	Serial.print(radio_no+1);
	Serial.println(" saved to memory");
}

void offRoutine(void)
{
	saveRadioMem(99);
	Serial.println("Streamer will shutdown in 3 seconds");
	tft.fillScreen(TFT_BLACK);
	tft.setTextColor(tft.color565(0,255,255),TFT_BLACK);
	tft.setCursor(35,60);
	tft.print("Shutting down in 3...");
	delay(1000);
	tft.fillScreen(TFT_BLACK);
	tft.setCursor(35,60);
	tft.print("Shutting down in 2...");
	delay(1000);
	tft.fillScreen(TFT_BLACK);
	tft.setCursor(35,60);
	tft.print("Shutting down in 1...");
	delay(1000);
	tft.fillScreen(TFT_BLACK);
	tft.setCursor(50,60);
	tft.print("Bye!");
	delay(1000);
	player.stop_mp3client();
	tft.fillScreen(TFT_BLACK);
	digitalWrite(17, LOW);
	gpio_hold_en(GPIO_NUM_17);
	gpio_deep_sleep_hold_en();
	esp_deep_sleep_start();
}

void checkIR(void)
{
	if (IrReceiver.decode())
	{
		if (IrReceiver.decodedIRData.decodedRawData == 0xF9067C87)  // This is my remote 'Forward' button, change it to yours
		{
			Serial.println(IrReceiver.decodedIRData.decodedRawData, HEX);
			Serial.println("IR: 'Forward'");
			radio_no++;
			if (radio_no < 0 || radio_no > radio_qty-1)	{radio_no = 0;}  // If array index is out of bounds, return to the beginning of the array
			connectiontoRadio();
			IrReceiver.resume();
		}
		if (IrReceiver.decodedIRData.decodedRawData == 0xFA057C87)  // This is my remote 'Back' button, change it to yours
		{
			Serial.println(IrReceiver.decodedIRData.decodedRawData, HEX);
			Serial.println("IR: 'Back'");
			radio_no--;
			if (radio_no < 0 || radio_no > radio_qty-1)	{radio_no = radio_qty-1;}  // If array index is out of bounds, return to the end of the array
			connectiontoRadio();
			IrReceiver.resume();
		}
		if (IrReceiver.decodedIRData.decodedRawData == 0x76897C87 || IrReceiver.decodedIRData.decodedRawData == 0x7A857C87 || IrReceiver.decodedIRData.decodedRawData == 0x3DC27C87 || IrReceiver.decodedIRData.decodedRawData == 0x22DD7C87 || IrReceiver.decodedIRData.decodedRawData == 0x728D7C87)  // These are my remote 'Other sources' button to shutdown the streamer, change it to yours
		{
			Serial.println(IrReceiver.decodedIRData.decodedRawData, HEX);
			offRoutine();
		}
		IrReceiver.resume();
	}
}

void checkSerialMonitor(void)
{
	if (Serial.available())
    {
		String r = Serial.readString();
		r.trim();
		if (r.length() > 3)  // If detects link
        {
			radio_no=-1;
			player.connecttohost(r);
		}
        else  // If detects number
        {
			radio_no = r.toInt();
			if (radio_no > 0 && radio_no <= radio_qty)
			{
				radio_no--;
				connectiontoRadio();
			}
			else
			{
				Serial.print("Please enter Radio number between 1 and ");
				Serial.println(radio_qty);
			}
        }
    }
}